/**
 * $Id: bmpfont.cc,v 1.2 2003/04/02 15:10:12 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "bmpfont.h"
#include "mysdl.h"

#include <cctype>

const double BMPFont::dotPerChar_ = 300.0 / 41;
const int BMPFont::charWidth_ = 7;
const int BMPFont::charHeight_ = 14;

BMPFont::BMPFont(const std::string& filename)
	: sample_("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ.<>^ ")
{
	SDL_Surface* tmp = SDL_LoadBMP(filename.c_str());
	font_ = SDL_DisplayFormat(tmp);
	SDL_FreeSurface(tmp);

	MySDL_SetColorKey(font_, SDL_SRCCOLORKEY|SDL_RLEACCEL, 0, 0, 0);
}

void BMPFont::render(const char* str, SDL_Surface* dstSurf, int x, int y) {
	static const double dotPerChar_ = 300.0 / 41;
	static const int charWidth_ = 7;
	static const int charHeight_ = 14;

	for (unsigned  int i = 0; *str; str++, i++) {
		char c = std::toupper(*str);
		unsigned int n = sample_.find(c);

		if (n == std::string::npos) continue;

		MySDL_BlitSurface(
			font_, (int)(n*dotPerChar_), 0, charWidth_, charHeight_,
			dstSurf, x+(int)(i*dotPerChar_), y);
	}
}

void BMPFont::renderColored(const char* str, struct SDL_Surface* surf,
							int x, int y, Uint32 color)
{
	SDL_Surface* tmp = MySDL_CreateRGBSurface(
		SDL_SWSURFACE, width(strlen(str)), height(), 8);
	tmp = MySDL_DisplayFormat(tmp);
	MySDL_SetColorKey(tmp, SDL_SRCCOLORKEY|SDL_RLEACCEL, 0,0,0);
	render(str, tmp, 0, 0);
	MySDL_ChangeColor(tmp, color);
	MySDL_BlitSurface(tmp, surf, x, y);
	SDL_FreeSurface(tmp);
}

int BMPFont::height() const {
	return charHeight_;
}

int BMPFont::width(int num) const {
	return static_cast<int>(dotPerChar_ * num);
}

