#include "bonus.h"

#include "score.h"
#include "resource.h"
#include "bpo.h"

#include <ctype.h>

Bonus::Bonus(std::string name)
	: name_(name), score_(0), count_(0)
{
	init();
}

void Bonus::init() {
	score_ = BPO::rc().get<int>("bonus"+name_+"Score");
	flash_ = BPO::rc().get<bool>("bonus"+name_+"Flash");

	label_ = name_;
	for (size_t i = 1; i < label_.length(); i++) {
		if (isupper(label_[i])) {
			label_.insert(i, " ");
			i++;
		}
	}
}

void Bonus::addScore(Score* score) {
	count_++;

	score->addBonus(score_);
}

BonusMgr::BonusMgr()
	: minuteSurvival_("MinuteSurvival"),
	  tenSecondsSurvival_("10SecondsSurvival"),
	  highSpeedDodge_("HighSpeedDodge"), lowSpeedDodge_("LowSpeedDodge"),
	  safetyDodge_("SafetyDodge"), riskyDodge_("RiskyDodge"),
	  justTiming_("JustTiming"), goodTiming_("GoodTiming")
{
	bonuses_.push_back(&minuteSurvival_);
	bonuses_.push_back(&tenSecondsSurvival_);
	bonuses_.push_back(&highSpeedDodge_);
	bonuses_.push_back(&lowSpeedDodge_);
	bonuses_.push_back(&safetyDodge_);
	bonuses_.push_back(&riskyDodge_);
	bonuses_.push_back(&justTiming_);
	bonuses_.push_back(&goodTiming_);
}

std::istream& operator>> (std::istream& is, Bonus& bonus) {
	is >> bonus.name_ >> bonus.score_ >> bonus.count_;
	bonus.init();
	return is;
}

std::ostream& operator<< (std::ostream& os, const Bonus& bonus) {
	os << bonus.name_ << " "<< bonus.score_ << " " << bonus.count_;
	return os;
}

BonusMgr::BonusMgr(std::istream& is) {
	is >> minuteSurvival_
	   >> tenSecondsSurvival_
	   >> highSpeedDodge_
	   >> lowSpeedDodge_
	   >> safetyDodge_
	   >> riskyDodge_
	   >> justTiming_
	   >> goodTiming_;
	bonuses_.push_back(&minuteSurvival_);
	bonuses_.push_back(&tenSecondsSurvival_);
	bonuses_.push_back(&highSpeedDodge_);
	bonuses_.push_back(&lowSpeedDodge_);
	bonuses_.push_back(&safetyDodge_);
	bonuses_.push_back(&riskyDodge_);
	bonuses_.push_back(&justTiming_);
	bonuses_.push_back(&goodTiming_);
}

std::ostream& operator<< (std::ostream& os, const BonusMgr& bonus) {
	for (size_t i = 0; i < bonus.bonuses_.size(); i++) {
		os << *bonus.bonuses_[i] << std::endl;
	}

	return os;
}


