/**
 * $Id: bonus.h,v 1.3 2003/04/06 13:34:57 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef bonus_h_
#define bonus_h_

#include <string>
#include <vector>
#include <iosfwd>

class Bonus {
public:
	Bonus() {}
	explicit Bonus(std::string name);

	const std::string& label() const { return label_; }
	int score() const { return score_; }
	int count() const { return count_; }
	bool flash() const { return flash_; }
	int allScore() const { return score_ * count_; }

	void addScore(class Score* score);

private:
	void init();

private:
	std::string name_;
	std::string label_;
	int score_;
	int count_;
	bool flash_;

private:
	friend std::istream& operator>> (std::istream& is, Bonus& bonus);
	friend std::ostream& operator<< (std::ostream& os, const Bonus& bonus);
};

class BonusMgr {
public:
	BonusMgr();
	explicit BonusMgr(std::istream& is);
	const std::vector<const Bonus*>& bonuses() { return bonuses_; }

	typedef std::vector<const Bonus*>::const_iterator iterator;

	Bonus& minuteSurvival() { return minuteSurvival_; }
	Bonus& tenSecondsSurvival() { return tenSecondsSurvival_; }
	Bonus& highSpeedDodge() { return highSpeedDodge_; }
	Bonus& lowSpeedDodge() { return lowSpeedDodge_; }
	Bonus& safetyDodge() { return safetyDodge_; }
	Bonus& riskyDodge() { return riskyDodge_; }
	Bonus& justTiming() { return justTiming_; }
	Bonus& goodTiming() { return goodTiming_; }

private:
	Bonus minuteSurvival_, tenSecondsSurvival_;
	Bonus highSpeedDodge_, lowSpeedDodge_;
	Bonus safetyDodge_, riskyDodge_;
	Bonus justTiming_, goodTiming_;

	std::vector<const Bonus*> bonuses_;

private:
	friend std::istream& operator>> (std::istream& is, BonusMgr& bonus);
	friend std::ostream& operator<< (std::ostream& os, const BonusMgr& bonus);
};

#endif // ! bonus_h_

