/**
 * $Id: bpo.cc,v 1.20 2003/07/24 10:53:10 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "bpo.h"

#include "cpputil.h"
#include "scene.h"
#include "resource.h"
#include "fpstimer.h"
#include "mymath.h"
#include "clear_screen.h"
#include "padinput.h"
#include "bulletml_holder.h"
#include "profile.h"
#include "bmpfont.h"
#include "music.h"

#include "config.h"
#ifdef USE_SDL_SOUND
# include <SDL_sound.h>
#endif

#include <SDL.h>

#include <memory>
#include <cstdlib>
#include <ctime>

#ifdef BORLAND
# include <windows.h>
# include <dir.h>
#endif // BORLAND

struct BPOImpl {
	BPOImpl(int argc, char* argv[]);
	void run();
	~BPOImpl();

	void init();

	void initResource(int argc, char* argv[]);
	void initMisc();
	void initSDL();

	void resetColors();

	void runScene();
	void runClear();

	void endCheck();

	void end();
	void quit();

	Resource rc;
	std::auto_ptr<Scene> scene;
	FPSTimer timer;

	SDL_Surface* screen;

	Input* input;

	std::auto_ptr<BulletMLHolder> bulletmls;

	std::auto_ptr<BMPFont> bmpFont;
	//Kanji_Font* bdfFont;

	int fpsDrawX_, fpsDrawY_;

	std::auto_ptr<MusicMgr> musicMgr_;

	int clearMode_;

};

BPOImpl::BPOImpl(int argc, char* argv[]) {
	initResource(argc, argv);
	initMisc();
	initSDL();
};

void BPOImpl::init() {

	input = generateInputFromConf("save/key.conf").release();

	bulletmls.reset(new BulletMLHolder);

	timer.setFps(60);

	scene.reset(SceneFactory::generateTitle().release());

	musicMgr_.reset(new MusicMgr());
}

void BPOImpl::initResource(int argc, char* argv[]) {
	rc.addDefault();
	rc.addResourceFile();
	rc.addArgument(argc, argv);
}

void BPOImpl::initMisc() {
	std::srand(std::time(0));

/*
	bdfFont = Kanji_OpenFont("data/fonts/K12-1.bdf", 12);
	Kanji_AddFont(bdfFont, "data/fonts/6x12.bdf");

#ifdef WINDOWS
	Kanji_SetCodingSystem(bdfFont, KANJI_SJIS);
#else
	Kanji_SetCodingSystem(bdfFont, KANJI_EUC);
#endif
*/

	fpsDrawX_ = rc.get<int>("fpsDrawX");
	fpsDrawY_ = rc.get<int>("fpsDrawY");

	clearMode_ = rc.get<int>("clearMode");
}

namespace {
	Uint8 convBright(Uint8 x) {
		int ret = (int)(std::sqrt((double)x/255)*255);
		if (ret < 0) ret = 0;
		else if (ret > 255) ret = 255;
		return ret;
	}
}

void BPOImpl::resetColors() {
	SDL_Color colors[256];
	for (int i = 0; i < 256; i++) {
		int r = rnd(100);
		if (i == 0 || i == 255 || r > 5) {
			colors[i].r = i;
			colors[i].g = i;
			colors[i].b = i;
		}
		else if (r == 0) {
			colors[i].r = 127;
			colors[i].g =   0;
			colors[i].b =   0;
		}
		else if (r == 1) {
			colors[i].r =   0;
			colors[i].g = 127;
			colors[i].b =   0;
		}
		else if (r == 2) {
			colors[i].r =   0;
			colors[i].g =   0;
			colors[i].b = 127;
		}
		else if (r == 3) {
			colors[i].r = 127;
			colors[i].g = 127;
			colors[i].b =   0;
		}
		else if (r == 4) {
			colors[i].r = 127;
			colors[i].g =   0;
			colors[i].b = 127;
		}
		else {
			colors[i].r =   0;
			colors[i].g = 127;
			colors[i].b = 127;
		}
	}
	colors[254].r =  255;
	colors[254].g =    0;
	colors[254].b =    0;
	colors[253].r =  255;
	colors[253].g =  255;
	colors[253].b =    0;
	colors[252].r =    0;
	colors[252].g =  255;
	colors[252].b =    0;
	colors[251].r =  200;
	colors[251].g =  200;
	colors[251].b =  255;
	colors[250].r =  255;
	colors[250].g =  127;
	colors[250].b =    0;
	colors[249].r =    0;
	colors[249].g =    0;
	colors[249].b =  255;
	colors[248].r =  255;
	colors[248].g =    0;
	colors[248].b =  255;
	colors[247].r =  127;
	colors[247].g =  127;
	colors[247].b =  127;
	colors[246].r =    0;
	colors[246].g =  255;
	colors[246].b =  255;
	colors[245].r =    0;
	colors[245].g =  127;
	colors[245].b =  255;
	colors[ 30].r =    0;
	colors[ 30].g =  127;
	colors[ 30].b =    0;
	colors[  4].r = 100+rnd(100);
	colors[  4].g = 100+rnd(100);
	colors[  4].b = 100+rnd(100);

	if (rc.get<bool>("bright")) {
		for (int i = 0; i < 245; i++) {
			colors[i].r = convBright(colors[i].r);
			colors[i].g = convBright(colors[i].g);
			colors[i].b = convBright(colors[i].b);
		}
	}

	if (0 == SDL_SetColors(screen, colors, 0, 256)) {
		std::cout << "some colors may be wrong." << std::endl;
	}
}

void BPOImpl::initSDL() {
	int ret = SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_NOPARACHUTE);
	check(ret == 0, SDL_GetError());

	screen = SDL_SetVideoMode(rc.get<Uint32>("screenWidth"),
							  rc.get<Uint32>("screenHeight"),
							  rc.get<Uint32>("screenBpp"),
							  rc.get<Uint32>("screenFlag"));
	check(screen != 0, SDL_GetError());

	resetColors();

	bmpFont.reset(new BMPFont("bmp/strings.bmp"));

#ifdef USE_SDL_SOUND
	if (!Sound_Init()) {
		error(std::string("sound init failed: ") + Sound_GetError());
	}
#endif
}

void BPOImpl::run() {
	timer.start();
	while (scene.get()) {
		runScene();
		if (!timer.shouldSkip()) {
			bmpFont->render(
				"FPS" + boost::lexical_cast<std::string>(timer.getTrueFps()),
				screen, fpsDrawX_, fpsDrawY_);
			runClear();
		}
		{
			timer.wait();
		}
		endCheck();
	}
}

void BPOImpl::endCheck() {
	SDL_Event ev;
	while (SDL_PollEvent(&ev)) {
		if (ev.type == SDL_QUIT) {
			quit();
		}
	}
	if (input->getButton(2)) end();
}

void BPOImpl::end() {
	scene.reset(scene->getEscapedNext().release());
	while (input->getButton(2)) {
		SDL_PumpEvents();
	}

	std::cout << "FPS: " << timer.getTrueFps()
			  << ", dropped frames: " << timer.getDroppedTurn() << std::endl;
	timer.start();
}

void BPOImpl::quit() {
	scene.reset(0);
}

void BPOImpl::runScene() {
	scene->run();
	if (scene->isEnd()) {
		scene.reset(scene->getNext().release());

		std::cout << "FPS: " << timer.getTrueFps()
				  << ", dropped frames: " << timer.getDroppedTurn()
				  << std::endl;
		timer.start();
	}
}

void BPOImpl::runClear() {
	SDL_Flip(screen);

	if (SDL_MUSTLOCK(screen)) SDL_LockSurface(screen);

	clearScreen(screen, clearMode_);

	if (SDL_MUSTLOCK(screen)) SDL_UnlockSurface(screen);
}

BPOImpl::~BPOImpl() {
#ifdef USE_SDL_SOUND
	Sound_Quit();
#endif
	SDL_Quit();
}

BPO::BPO(int argc, char* argv[])
	: pimpl_(new BPOImpl(argc, argv))
{
	spimpl_ = pimpl_.get();
	spimpl_->init();
}

void BPO::run() {
	pimpl_->run();
}

Resource& BPO::rc() {
	return spimpl_->rc;
}

SDL_Surface* BPO::getScreen() {
	return spimpl_->screen;
}

SDL_Surface* BPO::getSurface() {
	return spimpl_->screen;
}

bool BPO::shouldSkip() {
	return spimpl_->timer.shouldSkip();
}

Input* BPO::getInput() {
	return spimpl_->input;
}

class BulletMLParser* BPO::getNormalBullet() {
	return spimpl_->bulletmls->getNormalBullet();
}

class BulletMLParser* BPO::getEasyBullet() {
	return spimpl_->bulletmls->getEasyBullet();
}

class BulletMLParser* BPO::getHardBullet() {
	return spimpl_->bulletmls->getHardBullet();
}

void BPO::resetColors() {
	spimpl_->resetColors();
}

BMPFont* BPO::getBMPFont() {
	return spimpl_->bmpFont.get();
}

void BPO::timerStart() {
	spimpl_->timer.start();
}

int BPO::getFps() {
	return spimpl_->timer.getTrueFps();
}

const Music* BPO::getMusic() {
	return spimpl_->musicMgr_->getRandomMusic();
}

void BPO::setClearMode(int mode) {
	spimpl_->clearMode_ = mode;
}

void BPO::setBulletMode() {
	spimpl_->bulletmls->setMode(rc().get<std::string>("mode"));
}

BPOImpl* BPO::spimpl_ = 0;

int main(int argc, char* argv[]) {
	try {
#ifdef BORLAND
		std::string exe(argv[0]);
		size_t lastSlash = exe.find_last_of("\\");
		if (lastSlash != std::string::npos) {
			std::string dir(exe.substr(0, lastSlash));
			chdir(dir.c_str());
		}
#endif // BORLAND
		std::auto_ptr<BPO> bpo(new BPO(argc, argv));
		bpo->run();
	}
	catch (const std::exception& e) {
#ifdef BORLAND
        MessageBox(0,
				   (std::string("顼ǤƤϰʲ̤ꡢ") +
					std::string(e.what()) +
					std::string("̵顼Ȥĺȴ򤷤Ǥ")).c_str(),
				   "餻", MB_OK);
#endif
		std::cerr << e.what() << std::endl;
	}
	catch (...) {
#ifdef BORLAND
        MessageBox(0,
				   (std::string("褯狼ʤ顼Ǥ") +
					std::string("̵顼Ȥĺȴ򤷤Ǥ")).c_str(),
				   "餻", MB_OK);
#endif
		std::cerr << "unknown exception" << std::endl;
	}
	return 0;
}

