/**
 * $Id: bullet_command.cc,v 1.4 2003/07/24 10:53:10 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "bullet_command.h"

#include "game.h"
#include "player.h"
#include "bullet.h"
#include "mymath.h"
#include "bpo.h"
#include "resource.h"

#include "bulletml/bulletmlrunner.h"

BulletCommand::BulletCommand(Game* game,
							 BulletMLState* state, Bullet* bullet)
	: BulletMLRunner(state), game_(game), bullet_(bullet),
	  shotSpeedRatio_(BPO::rc().get<double>("shotSpeed"))
{}

#define SRM(val) ((shotSpeedRatio_ == 1.0) ? val : val * shotSpeedRatio_)
#define SRD(val) ((shotSpeedRatio_ == 1.0) ? val : val / shotSpeedRatio_)

double BulletCommand::getBulletDirection() {
	return rtod(angle(bullet_->sx(), bullet_->sy()));
}

double BulletCommand::getAimDirection() {
	return rtod(game_->getPlayer()->angle(*bullet_));
}

double BulletCommand::getBulletSpeed() {
    return SRD(length(bullet_->sx(), bullet_->sy()));
}

double BulletCommand::getDefaultSpeed() {
    return 2;
}

double BulletCommand::getRank() {
    return static_cast<double>(game_->getRank()) / 100;
}

void BulletCommand::createSimpleBullet(double direction, double speed) {
	game_->addShot(bullet_->x(), bullet_->y(), dtor(direction), SRM(speed));
}

void BulletCommand::createBullet(BulletMLState* state,
								 double direction, double speed) {
	game_->addBullet(state, bullet_->x(), bullet_->y(),
					 dtor(direction), SRM(speed));
}

int BulletCommand::getTurn() {
    return game_->getTurn();
}

void BulletCommand::doVanish() {
	bullet_->setDead();
}


void BulletCommand::doChangeDirection(double dir) {
	double spd = getBulletSpeed();
	bullet_->setSpeedX(spd*std::sin(dtor(dir)));
	bullet_->setSpeedY(-spd*std::cos(dtor(dir)));
}

void BulletCommand::doChangeSpeed(double spd) {
	double dir = angle(bullet_->sx(), bullet_->sy());
	spd = SRM(spd);
	bullet_->setSpeedX(spd*std::sin(dir));
	bullet_->setSpeedY(-spd*std::cos(dir));
}

void BulletCommand::doAccelX(double x) {
	x = SRM(x);
	bullet_->setSpeedX(x);
}

void BulletCommand::doAccelY(double y) {
	y = SRM(y);
	bullet_->setSpeedY(y);
}

double BulletCommand::getBulletSpeedX() {
    return SRD(bullet_->sx());
}

double BulletCommand::getBulletSpeedY() {
    return SRD(bullet_->sy());
}

