#include "bulletml_holder.h"

#include "bpo.h"
#include "resource.h"
#include "mymath.h"
#include "dir_ite.h"

#include "bulletml/bulletmlparser.h"
#include "bulletml/bulletmlparser-tinyxml.h"
#include "bulletml/bulletmlparser-ygg.h"

#ifdef USE_XERCES
# include "bulletml/bulletmlparser-xerces.h"
#endif

#include "sgi/select.h"

#include <dirent.h>

#include <algorithm>
#include <functional>

extern unsigned long genrand_int32(void);
// from rRootage's genmcr.h
#define randN(N) ((int)(genrand_int32()>>5)%(N))

BulletMLHolder::BulletMLHolder() {
	Dir bulletDir(BPO::rc().get<std::string>("bulletDir"));
	loadDir(bulletDir.full("bpo")+"/normal", mode2normals_["bpo"]);
	loadDir(bulletDir.full("bpo")+"/easy", mode2easys_["bpo"]);
	loadDir(bulletDir.full("bpo")+"/hard", mode2hards_["bpo"]);
	for (Dir::iterator ite = bulletDir.begin();
		 ite != bulletDir.end(); ++ite)
	{
		size_t pos = ite->find(".add");
		if (pos == std::string::npos) continue;
		std::string modeName = *ite;
		modeName.erase(pos);
		mode2normals_[modeName] = mode2normals_["bpo"];
		mode2easys_[modeName] = mode2easys_["bpo"];
		mode2hards_[modeName] = mode2hards_["bpo"];
		loadDir(bulletDir.full(*ite)+"/normal", mode2normals_[modeName]);
		loadDir(bulletDir.full(*ite)+"/easy", mode2easys_[modeName]);
		loadDir(bulletDir.full(*ite)+"/hard", mode2hards_[modeName]);
	}
}

void BulletMLHolder::setMode(std::string mode) {
	setMode_(mode2normals_[mode], &normal_);
	setMode_(mode2easys_[mode], &easy_);
	setMode_(mode2hards_[mode], &hard_);
}

void BulletMLHolder::setMode_(const Name2Bullet_& inb, Bullets_* outb)
{
	outb->clear();
	std::transform(inb.begin(), inb.end(),
				   std::back_inserter(*outb),
				   sgi::select2nd<Name2Bullet_::value_type>());
}

#include "profile.h"

void BulletMLHolder::loadDir(const std::string& dirname,
							 Name2Bullet_& bullets) {
	PROFILE_INIT;

	DIR* dir = opendir(dirname.c_str());
	while (dirent* d = readdir(dir)) {
		std::string name(dirname + "/" + d->d_name);
		if (name.find(".xml") == name.size() - 4) {
			std::cout << "xml: " << d->d_name << " - " << name << std::endl;

#ifdef USE_XERCES
			PROFILE("xerces");
			BulletMLParser* bulletml = new BulletMLParserXerces(name);
#else
			PROFILE("tinyxml");
			BulletMLParser* bulletml = new BulletMLParserTinyXML(name);
#endif
			bulletml->build();
			bullets[std::string(d->d_name)] = bulletml;
			deleteList_.push_back(bulletml);
		}
		else if (name.find(".sox") == name.size() - 4) {
			std::cout << "sox: " << d->d_name << " - " << name << std::endl;

			PROFILE("sox");
			BulletMLParser* bulletml = new BulletMLParserYggdrasil(name);
			bulletml->build();
			bullets[std::string(d->d_name)] = bulletml;
			deleteList_.push_back(bulletml);
		}
	}

	PROFILE_OUTPUT;
}

BulletMLHolder::~BulletMLHolder() {
	delete_clear(deleteList_);
}

BulletMLParser* BulletMLHolder::getNormalBullet() {
	return normal_[randN(normal_.size())];
}

BulletMLParser* BulletMLHolder::getEasyBullet() {
	return easy_[randN(easy_.size())];
}

BulletMLParser* BulletMLHolder::getHardBullet() {
	return hard_[randN(hard_.size())];
}
