#include "clear_screen.h"

#include <SDL.h>
#include <SDL_imageFilter.h>

#define CORE_PLUS(algo) \
pf = (Uint8*)surf->pixels; \
for (int j = 0; j < surf->h; j++) { \
	p = pf; \
	for (int i = 0; i < surf->w; i++) { \
        algo \
		p++; \
	} \
	pf += surf->pitch; \
} \
break;

#define CORE_MINUS(algo) \
pf = (Uint8*)surf->pixels + surf->h * surf->pitch + surf->w; \
for (int j = 0; j < surf->h; j++) { \
	pf -= surf->pitch; \
	p = pf; \
	for (int i = 0; i < surf->w; i++) { \
		p--; \
		algo \
	} \
} \
break;


void clearScreen(SDL_Surface* surf, int mode) {
	static int submode = 0;
	submode = (submode + 1) & 3;

	Uint8 prev = 0;
	Uint8* pf;
	Uint8* p;

	switch (mode) {
	case 0:
	{
		switch (submode) {
		case 0:
			CORE_PLUS(*p += prev; *p >>= 1; prev = *p;);
		case 1:
			CORE_MINUS(*p += prev; *p >>= 1; prev = *p;);
		case 2:
			CORE_PLUS(*p += prev; *p >>= 1; prev = *p >> 1;);
		case 3:
			CORE_MINUS(*p += prev; *p >>= 1; prev = *p >> 1;);
		}
		break;
	}
	case 1:
	{
		switch (submode) {
		case 0:
			CORE_PLUS(*p >>= 1;	*p += prev;	prev = *p >> 1;);
		case 1:
			CORE_MINUS(*p >>= 1; *p += prev; prev = *p >> 1;);
		case 2:
			CORE_PLUS(*p += prev; *p >>= 1; prev = *p >> 1;);
		case 3:
			CORE_MINUS(*p += prev; *p >>= 1; prev = *p >> 1;);
		}
		break;
	}
	case 2:
	{
		switch (submode) {
		case 0:
			CORE_PLUS(*p += prev; *p >>= 1; prev = *p;);
		case 1:
			CORE_MINUS(*p += prev; *p >>= 1; prev = *p;);
		}
		break;
	}
	case 3:
	{
		SDL_imageFilterMMXon();
		switch (submode) {
		case 0:
		case 1:
		{
			p = (Uint8*)surf->pixels;
			Uint8 p2[450];
			for (int j = 0; j < surf->h; j++) {
				SDL_imageFilterShiftRight(p, p2+1, 449, 1);
				SDL_imageFilterAdd(p, p2, p, 450);
				SDL_imageFilterShiftRight(p, p, 450, 1);
				p += surf->pitch;
			}
			break;
		}
		case 2:
		case 3:
		{
			p = (Uint8*)surf->pixels;
			Uint8 p2[450];
			for (int j = 0; j < surf->h; j++) {
				SDL_imageFilterShiftRight(p+1, p2, 449, 1);
				SDL_imageFilterAdd(p, p2, p, 450);
				SDL_imageFilterShiftRight(p, p, 450, 1);
				p += surf->pitch;
			}
			break;
		}
		}
		SDL_imageFilterMMXoff();
		break;
	}
	case 4:
	{
		SDL_imageFilterMMXon();
		Uint8 p2[450];
		switch (submode) {
		case 0:
		case 1:
		{
			p = (Uint8*)surf->pixels;
			for (int j = 0; j < surf->h; j+=2) {
				SDL_imageFilterShiftRight(p, p2+1, 449, 1);
				SDL_imageFilterAdd(p, p2, p, 450);
				SDL_imageFilterShiftRight(p, p, 450, 1);
				p += surf->pitch * 2;
			}
		}
		case 2:
		case 3:
		{
			p = (Uint8*)surf->pixels + surf->pitch;
			for (int j = 1; j < surf->h; j+=2) {
				SDL_imageFilterShiftRight(p+1, p2, 449, 1);
				SDL_imageFilterAdd(p, p2, p, 450);
				SDL_imageFilterShiftRight(p, p, 450, 1);
				p += surf->pitch * 2;
			}
			break;
		}
		}
		SDL_imageFilterMMXoff();
		break;
	}
	case 5:
	{
		SDL_imageFilterMMXon();
		Uint8 prevn[450];
		for (int i = 0; i < 450; i++) prevn[i] = 0;
		Uint8 andn[450];
		for (int i = 0; i < 450; i++) andn[i] = 63;

		p = (Uint8*)surf->pixels + (surf->h - 1) * surf->pitch;
		for (int j = 0; j < surf->h; j++) {
			SDL_imageFilterAdd(p, prevn, p, 450);
			SDL_imageFilterShiftRight(p, p, 450, 1);
			SDL_imageFilterBitAnd(p, andn, prevn, 450);
			p -= surf->pitch;
		}
		SDL_imageFilterMMXoff();

		break;
	}
	case 6:
	{
		SDL_imageFilterMMXon();
		Uint8 prevn[450];
		for (int i = 0; i < 450; i++) prevn[i] = 0;

		switch (submode) {
		case 1:
		case 3:
			p = (Uint8*)surf->pixels;
			for (int j = 0; j < surf->h; j++) {
				SDL_imageFilterAdd(p, prevn, p, 450);
				SDL_imageFilterShiftRight(p, p, 450, 1);
				for (int i = 0; i < 450; i++) prevn[i] = p[i];
				p += surf->pitch;
			}
			break;
		case 0:
		case 2:
			p = (Uint8*)surf->pixels + (surf->h - 1) * surf->pitch;
			for (int j = 0; j < surf->h; j++) {
				SDL_imageFilterAdd(p, prevn, p, 450);
				SDL_imageFilterShiftRight(p, p, 450, 1);
				for (int i = 0; i < 450; i++) prevn[i] = p[i];
				p -= surf->pitch;
			}
			break;
		}
		SDL_imageFilterMMXoff();
		break;
	}
	case 7:
	{
		SDL_imageFilterMMXon();
		Uint8 prevn[450];
		for (int i = 0; i < 450; i++) prevn[i] = 0;

		p = (Uint8*)surf->pixels + (surf->h - 1) * surf->pitch;
		for (int j = 0; j < surf->h; j++) {
			SDL_imageFilterAdd(p, prevn, p, 450);
			SDL_imageFilterShiftRight(p, p, 450, 1);
			for (int i = 0; i < 450; i++) prevn[i] = p[i];
			p -= surf->pitch;
		}

		SDL_imageFilterMMXoff();
		break;
	}
	case 8:
	{
		SDL_imageFilterMMXon();
		switch (submode) {
		case 0:
		case 1:
		{
			p = (Uint8*)surf->pixels;
			Uint8 p2[450];
			for (int j = 1; j < surf->h; j++) {
				Uint8* prev = p;
				p += surf->pitch;
				SDL_imageFilterShiftRight(p, p2+1, 449, 1);
				SDL_imageFilterAdd(p, p2, prev, 450);
				SDL_imageFilterShiftRight(prev, prev, 450, 1);
			}
			break;
		}
		case 2:
		case 3:
		{
			p = (Uint8*)surf->pixels;
			Uint8 p2[450];
			for (int j = 1; j < surf->h; j++) {
				Uint8* prev = p;
				p += surf->pitch;
				SDL_imageFilterShiftRight(p+1, p2, 449, 2);
				SDL_imageFilterAdd(p, p2, prev, 450);
				SDL_imageFilterShiftRight(p, p, 450, 1);
			}
			break;
		}
		}
		SDL_imageFilterMMXoff();
		break;
	}
	default:
	{
		SDL_FillRect(surf, 0, 0);
	}

	}
}

