/**
 * $Id: chairman.cc,v 1.2 2003/04/02 15:10:14 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "chairman.h"

#include "reflection.h"
#include "oblivion.h"
#include "veto.h"
#include "position.h"
#include "roundtrip.h"

#include "bpo.h"
#include "mysdl.h"

#include "cpuutil.h"
#include "profile.h"

#include <SDL_gfxPrimitives.h>

#include <boost/functional.hpp>

#include <algorithm>
#include <functional>
#include <cassert>
#include <fstream>

ChairmanCpu::ChairmanCpu() {
	name_ = "chairman";

	std::ifstream is("save/cpu.conf");
	std::string str;
	while (is >> str) {
		if (str == "veto") diet_.push_back(new VetoCpu);
		else if (str == "reflection") diet_.push_back(new ReflectionCpu);
		else if (str == "oblivion") diet_.push_back(new OblivionCpu);
		else if (str == "position") diet_.push_back(new PositionCpu);
		else if (str == "roundtrip") diet_.push_back(new RoundtripCpu);
/*
		else if (str == "inspection") diet_.push_back(new InspectionCpu);
		else if (str == "human") diet_.push_back(new HumanCpu);
*/
	}
	assert(diet_.size() <= MAX_DIETS);

	surf_ = BPO::getSurface();
	cols_[0] = MySDL_Color(255, 0, 0, 255);
	cols_[1] = MySDL_Color(255, 127, 0, 255);
	cols_[2] = MySDL_Color(255, 255, 0, 255);
	cols_[3] = MySDL_Color(0, 255, 0, 255);
	cols_[4] = MySDL_Color(0, 0, 255, 255);
	cols_[5] = MySDL_Color(255, 0, 255, 255);
	cols_[6] = MySDL_Color(127, 127, 127, 255);
	cols_[7] = MySDL_Color(0, 255, 255, 255);
	cols_[8] = MySDL_Color(0, 127, 255, 255);
}

ChairmanCpu::~ChairmanCpu() {
	delete_clear(diet_);
}

void ChairmanCpu::calc() {
	PROFILE(name().c_str());

	for (size_t j = 0; j < diet_.size(); j++) {
		CpuInputBase* cpu = diet_[j];

		PROFILE(cpu->name().c_str());

		cpu->update();

		for (int i = 0; i < 9; i++) {
			Axis axis = Axis::createFromSmallCode(i);

			int eval = cpu->getAxisEvaluation(axis) * cpu->getConfidence();
			evaluations_[i] += eval;
			dietEvals_[j][i] = eval;
		}
	}
	int index = std::max_element(evaluations_, evaluations_+9) - evaluations_;

	axis_ = Axis::createFromSmallCode(index);

	decision_ = false;

	drawGraph();
}

void ChairmanCpu::registShot(const PosSpd& shot) {
	for (size_t i = 0; i < diet_.size(); i++) {
		diet_[i]->registShot(shot);
	}
}

void ChairmanCpu::registEnemy(const class Enemy* enemy) {
	for (size_t i = 0; i < diet_.size(); i++) {
		diet_[i]->registEnemy(enemy);
	}
}

void ChairmanCpu::report() const {
	for (size_t i = 0; i < diet_.size(); i++) {
		diet_[i]->report();
	}
}

/*
void ChairmanCpu::registEnemy(const Enemy* enemy) {
	for (size_t i = 0; i < diet_.size(); i++) {
		diet_[i]->registEnemy(enemy);
	}
}
*/

void ChairmanCpu::drawGraph() {
	if (SDL_MUSTLOCK(surf_)) SDL_LockSurface(surf_);

	for (size_t i = 0; i < diet_.size(); i++) {
		drawSingleGraph(dietEvals_[i], cols_[i]);
	}

	if (SDL_MUSTLOCK(surf_)) SDL_UnlockSurface(surf_);
}

void ChairmanCpu::drawSingleGraph(int* evals, SDL_Color col) {
	static const int cx = surf_->w / 2;
	static const int cy = surf_->h / 2;

	int ev = (evals[8] / 20 + 20) * 3;
	if (ev > cx-1) ev = cx-1;
	else if (ev < 0) ev = 0;
	int px = static_cast<int>(axis2vec[8].x * ev) + cx;
	int py = static_cast<int>(axis2vec[8].y * ev) + cy;

	for (size_t i = 1; i < 9; i++) {
		int ev = (evals[i] / 20 + 20) * 3;
		if (ev > cx-1) ev = cx-1;
		else if (ev < 0) ev = 0;
		int x = static_cast<int>(axis2vec[i].x * ev) + cx;
		int y = static_cast<int>(axis2vec[i].y * ev) + cy;

		lineRGBA(surf_, px + rnd(5), py + rnd(5),
				 x + rnd(5), y + rnd(5), col.r, col.g, col.b, col.unused);

		px = x;
		py = y;
	}
}
