/**
 * $Id: enemy_command.cc,v 1.3 2003/07/24 10:53:10 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "enemy_command.h"

#include "cpputil.h"
#include "game.h"
#include "player.h"
#include "enemy.h"
#include "mymath.h"
#include "bpo.h"
#include "resource.h"

EnemyCommand::EnemyCommand(Game* game, BulletMLParser* bulletml, Enemy* enemy)
	: BulletMLRunner(bulletml), game_(game), enemy_(enemy),
	  shotSpeedRatio_(BPO::rc().get<double>("shotSpeed"))
{}

#define SRM(val) ((shotSpeedRatio_ == 1.0) ? val : val * shotSpeedRatio_)
#define SRD(val) ((shotSpeedRatio_ == 1.0) ? val : val / shotSpeedRatio_)

double EnemyCommand::getBulletDirection() {
	// dummy
	return 0;
}

double EnemyCommand::getAimDirection() {
    return rtod(game_->getPlayer()->angle(*enemy_));
}

double EnemyCommand::getBulletSpeed() {
	// dummy
    return 0;
}

double EnemyCommand::getDefaultSpeed() {
    return 2;
}

double EnemyCommand::getRank() {
    return 0.5;
}

void EnemyCommand::createSimpleBullet(double direction, double speed) {
	game_->addShot(enemy_->x(), enemy_->y(), dtor(direction), SRM(speed));
}

void EnemyCommand::createBullet(BulletMLState* state,
								double direction, double speed)
{
	game_->addBullet(state, enemy_->x(), enemy_->y(), dtor(direction), SRM(speed));
}

int EnemyCommand::getTurn() {
    return game_->getTurn();
}

void EnemyCommand::doVanish() {
	// dummy
    return;
}
