/**
 * $Id: music.cc,v 1.7 2003/07/22 01:25:07 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "music.h"

#include "bpo.h"
#include "resource.h"

#include "cpputil.h"
#include "mymath.h"
#include "dir_ite.h"

/*
#include "mp3info/mp3info.hxx"
*/

#include <cctype>
#include <fstream>
#include <iostream>

#ifdef BORLAND
#include <windows.h>
#include <comdef.h>
#include <comlite.h>
#include <shlguid.h>
#include <shldisp.h>
#include <shlobj.h>
#endif

/*
class MyMpfile : public Mpfile {
public:
	unsigned long getLength() const {
		return length;
	}
};
*/

bool Music::hasSuffix(const std::string& str, const std::string& suf) {
	return str.rfind(suf) == str.size() - suf.size();
}

#ifdef BORLAND
//*********************************************************
// LoadShellLink
//*********************************************************
static BOOL LoadShellLink( IShellLink *psl, const WCHAR *wcLinkName, DWORD dwMode )
{
    HRESULT       hResult;
    IPersistFile *ppf;
    
    if ( S_OK != psl->QueryInterface( IID_IPersistFile, (void **)&ppf ) )
        return FALSE;
    
    hResult = ppf->Load( wcLinkName, dwMode );
    ppf->Release();
    return S_OK == hResult;
}//LoadShellLink

/// from http://www.hidecnet.ne.jp/~sinzan/tips/win/win_tip0c.htm
//*********************************************************
// 硼ȥåȤΥ
//     LinkFile ġġ 硼ȥåȥե̾(*.lnk)
//   TargetFile ġġ 
//          wfd ġġ ξʥ°ʤɡ
//*********************************************************
static BOOL GetTargetFile( const char *LinkFile, char *TargetFile, WIN32_FIND_DATA *wfd )
{
	IShellLink *psl;
	WCHAR       wcLinkFile[MAX_PATH+1];

	CoInitialize( NULL );
	if ( S_OK == CoCreateInstance( CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, IID_IShellLink, (void **)&psl ) )
	{
		MultiByteToWideChar( CP_ACP, 0, LinkFile, -1, wcLinkFile, MAX_PATH );
		if ( LoadShellLink( psl, wcLinkFile, STGM_READ ) ) // 硼ȥåȤΥ
		{
			psl->GetPath( TargetFile, MAX_PATH, wfd, SLGP_UNCPRIORITY ); // 

			psl->Release();
			CoUninitialize();
			return TRUE; // 
		}
		psl->Release();
	}
	CoUninitialize();
	return FALSE; // 
}//GetTargetFile
#endif // BORLAND

Music::Music(const Music& rhs)
	: filename_(rhs.filename_), name_(rhs.name_), artist_(rhs.artist_),
	  genre_(rhs.genre_), comment_(rhs.comment_),
	  bitrate_(rhs.bitrate_), sfreq_(rhs.sfreq_), time_(rhs.time_),
	  valid_(rhs.valid_)
{}

Music::Music(const std::string& filename)
	: filename_(filename), name_(filename),
	  artist_("unknown"), genre_("unknown"), comment_("nothing"),
	  bitrate_(0), sfreq_(0), time_(0),
	  valid_(true)
{
	size_t lastSlash = name_.rfind("/");
	size_t lastBSlash = name_.rfind("\\");
	if (lastSlash == std::string::npos) {
		if (lastBSlash != std::string::npos) {
			name_ = name_.substr(lastBSlash+1);
		}
	}
	else {
		if (lastBSlash != std::string::npos) {
			name_ = name_.substr(std::max(lastSlash, lastBSlash)+1);
		}
		else {
			name_ = name_.substr(lastSlash+1);
		}
	}

	bool cont;

	do {
		cont = false;

		std::ifstream is(filename_.c_str());
		if (!is.is_open()) {
			valid_ = false;
			std::cout << filename_ << ": not found." << std::endl;
			return;
		}

		if (hasSuffix(filename_, ".mp3") || hasSuffix(filename_, ".MP3")) {
/* ʾϤȤꤢαޤ
			MyMpfile file;
			file.mpopen(const_cast<char*>(filename_.c_str()), false);
			file.initfile(true);

			if (file.hastag) {
				if (file.tag->title[0]) name_ = file.tag->title;
				if (file.tag->artist[0]) artist_ = file.tag->artist;
				if (file.tag->genre[0]) genre_ = file.tag->genre;
				if (file.tag->comment[0]) comment_ = file.tag->comment;
			}

			if (file.haslayer) {
				bitrate_ = file.layer->bitrate();
				sfreq_ = file.layer->sfreq();
			}

			time_ = file.getLength();

			file.mpclose();
*/
		}
		else if (hasSuffix(filename_, ".wav") || hasSuffix(filename_, ".WAV"))
		{
		}
		else if (hasSuffix(filename_, ".mod") || hasSuffix(filename_, ".MOD"))
		{
		}
		else if (hasSuffix(filename_, ".xm")||hasSuffix(filename_, ".XM") ||
				 hasSuffix(filename_, ".669")||hasSuffix(filename_, ".669") ||
				 hasSuffix(filename_, ".amf")||hasSuffix(filename_, ".AMF") ||
				 hasSuffix(filename_, ".dsm")||hasSuffix(filename_, ".DSM") ||
				 hasSuffix(filename_, ".far")||hasSuffix(filename_, ".FAR") ||
				 hasSuffix(filename_, ".gdm")||hasSuffix(filename_, ".GDM") ||
				 hasSuffix(filename_, ".imf")||hasSuffix(filename_, ".IMF") ||
				 hasSuffix(filename_, ".it")||hasSuffix(filename_, ".IT") ||
				 hasSuffix(filename_, ".m15")||hasSuffix(filename_, ".M15") ||
				 hasSuffix(filename_, ".med")||hasSuffix(filename_, ".MED") ||
				 hasSuffix(filename_, ".mtm")||hasSuffix(filename_, ".MTM") ||
				 hasSuffix(filename_, ".okt")||hasSuffix(filename_, ".OKT") ||
				 hasSuffix(filename_, ".s3m")||hasSuffix(filename_, ".S3M") ||
				 hasSuffix(filename_, ".stm")||hasSuffix(filename_, ".STM") ||
				 hasSuffix(filename_, ".stx")||hasSuffix(filename_, ".STX") ||
				 hasSuffix(filename_, ".ult")||hasSuffix(filename_, ".ULT") ||
				 hasSuffix(filename_, ".uni")||hasSuffix(filename_, ".UNI"))
		{
		}
		else if (hasSuffix(filename_, ".aiff") || hasSuffix(filename_, ".AIFF"))
		{
		}
		else if (hasSuffix(filename_, ".au") || hasSuffix(filename_, ".AU"))
		{
		}
		else if (hasSuffix(filename_, ".fla") || hasSuffix(filename_, ".FLA")||
				 hasSuffix(filename_, ".flac") || hasSuffix(filename_, ".FLAC"))
		{
		}
		else if (hasSuffix(filename_, ".shn") || hasSuffix(filename_, ".SHN"))
		{
		}
		else if (hasSuffix(filename_, ".voc") || hasSuffix(filename_, ".VOC"))
		{
		}
		else if (hasSuffix(filename_, ".raw") || hasSuffix(filename_, ".RAW"))
		{
		}
#ifndef BORLAND
		else if (hasSuffix(filename_, ".mid") ||
				 hasSuffix(filename_, ".MID") ||
				 hasSuffix(filename_, ".midi") ||
				 hasSuffix(filename_, ".MIDI"))
		{
		}
#endif
		else if (hasSuffix(filename_, ".mpg") ||
				 hasSuffix(filename_, ".MPG") ||
				 hasSuffix(filename_, ".mpeg") ||
				 hasSuffix(filename_, ".MPEG"))
		{
		}
		else if (hasSuffix(filename_, ".ogg") || hasSuffix(filename_, ".OGG"))
		{
		}
#ifdef BORLAND
		else if (hasSuffix(filename_, ".lnk") || hasSuffix(filename_, ".LNK"))
		{
			char trueFile[2048];
			WIN32_FIND_DATA wfd;
			GetTargetFile(filename_.c_str(), trueFile, &wfd);
			filename_ = trueFile;
			cont = true;
		}
#endif
		else {
			valid_ = false;
		}
	} while (cont);

}

void Music::print() const {
	std::cout << "filename:\t" << filename_ << "\n"
			  << "name:\t" << name_ << "\n"
			  << "artist:\t" << artist_ << "\n"
			  << "genre:\t" << genre_ << "\n"
			  << "comment_:\t" << comment_ << "\n"
			  << "time:\t" << time_ << "\n";
}

MusicMgr::MusicMgr() {
	std::string song = BPO::rc().get<std::string>("song");
	if (song.find(".") == std::string::npos) {
		song.erase(song.size() - 1);
		Dir dir(song);
		for (Dir::iterator ite = dir.begin(); ite != dir.end(); ++ite) {
			Music* music = new Music(dir.full(*ite));
			if (music->isValid()) {
				musics_.push_back(music);
				std::cout << "music: " << music->name() << std::endl;
			}
			else {
				delete music;
			}
		}
		check(!musics_.empty(),
			  song + " directory doesn't contain valid music.");
	}
	else {
		Music* music = new Music(song);
		check(music->isValid(), song + ": not supported format.");
		musics_.push_back(music);
		std::cout << "music: " << music->name() << std::endl;
	}
}

MusicMgr::~MusicMgr() {
	delete_clear(musics_);
}

const Music* MusicMgr::getRandomMusic() const {
	return musics_[rnd(musics_.size())];
}

Music::Music(std::istream& is) {
	std::ws(is);
	char buf[2048];
	is.getline(buf, 2048);
	filename_ = buf;
	is.getline(buf, 2048);
	name_ = buf;
	is.getline(buf, 2048);
	artist_ = buf;
	is.getline(buf, 2048);
	genre_ = buf;
	is.getline(buf, 2048);
	comment_ = buf;
	is >> bitrate_;
	is >> sfreq_;
	is >> time_;
	valid_ = true;
}

std::ostream& operator<< (std::ostream& os, const Music& music) {
	os << music.filename_ << std::endl
	   << music.name_ << std::endl
	   << music.artist_ << std::endl
	   << music.genre_ << std::endl
	   << music.comment_ << std::endl
	   << music.bitrate_ << std::endl
	   << music.sfreq_ << std::endl
	   << music.time_ << std::endl;

	return os;
}


