/**
 * $Id: music.h,v 1.2 2003/04/06 13:34:57 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef music_h_
#define music_h_

#include <string>
#include <vector>
#include <iosfwd>

class Music {
public:
	explicit Music(const std::string& filename);
	explicit Music(std::istream& is);
	Music() {}
	Music(const Music& rhs);

	const std::string& filename() const { return filename_; }
	const std::string& name() const { return name_; }
	int time() const { return time_; }

	bool isValid() const { return valid_; }

	void print() const;

protected:
	void setName(const std::string& name) { name_ = name; }
	void setTime(int time) { time_ = time; }

private:
	bool hasSuffix(const std::string& str, const std::string& suf);

private:
	std::string filename_;
	std::string name_;
	std::string artist_;
	std::string genre_;
	std::string comment_;
	int bitrate_;
	int sfreq_;
	unsigned long time_;

	bool valid_;

private:
	friend std::ostream& operator<< (std::ostream& os, const Music& music);

};

class MusicMgr {
public:
	MusicMgr();
	~MusicMgr();

	const Music* getRandomMusic() const;

private:
	std::vector<Music*> musics_;

};

#endif // ! music_h_

