/**
 * $Id: mymath.h,v 1.8 2003/04/05 06:54:19 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef mymath_h_
#define mymath_h_

#include <cmath>
#include <cstdlib>

#include "config.h"

#ifndef M_PI
const double M_PI  = 3.14159265358979323846;
#endif
const double PI = M_PI;
const double PI2 = M_PI * 2;
const double PI3_2 = M_PI / 2 * 3;
const double PI_2 = M_PI / 2;

const double SQRT_2 = std::sqrt(2.0);

inline int rnd(int x) {
	return static_cast<int>((float)std::rand()*x/RAND_MAX);
}
inline unsigned int rnd(unsigned int x) {
	return static_cast<unsigned int>((float)std::rand()*x/RAND_MAX);
}
inline Num rnd(Num x) {
	return (Num)std::rand()*x/RAND_MAX;
}

inline Num dtor(Num d) {
	return PI * d / 180;
}
inline Num rtod(Num d) {
	return d / PI * 180;
}

inline Num length2(Num x, Num y) {
	return x*x+y*y;
}

inline Num length(Num x, Num y) {
	return std::sqrt(length2(x, y));
}

inline Num distance2(Num x1, Num y1, Num x2, Num y2) {
	return length2(x2-x1, y2-y1);
}

inline Num distance(Num x1, Num y1, Num x2, Num y2) {
	return std::sqrt(distance2(x1, y1, x2, y2));
}

inline Num angle(Num x, Num y) {
	if (y == 0) {
		if (x > 0) return PI_2;
		else return PI3_2;
	}
	else {
		Num ret = std::atan(x/y);
		return (y > 0) ? PI-ret : -ret;
	}
}

#endif // ! mymath_h_

