/**
 * $Id: orchestra.h,v 1.10 2003/07/24 10:53:10 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef orchestra_h_
#define orchestra_h_

#include <vector>
#include <string>
#include <set>

#include "config.h"

#ifdef USE_SDL_SOUND
# include <SDL_sound.h>
#else
# include <SDL_mixer.h>
#endif

class Orchestra {
public:
	Orchestra(const std::string& musicFile, class Game* game);
	~Orchestra();

	void start();
	void end();
	bool isEnd();

	void callback(Uint8* stream, int len);

private:
	void resetBullets();

	void initSound(const std::string& musicFile);

private:
#ifdef USE_SDL_SOUND
	Sound_Sample* sample;
#else
	Mix_Music* music_;
#endif

	class Game* game_;

	int prev_;

	int seed_;

	class BulletMLParser* normalMainBullet_;
	class BulletMLParser* normalSubBullet_;
	class BulletMLParser* hardBullet_;

	int notBulletTurn_;
private:
	static const int FREQ_NUM = 512;
	static const int FREQ_NUM_2 = FREQ_NUM >> 1;
	static const int SPECTRU_SIZE = 256;

	static const int PEEK_INT = 7;
	static const int FORMANT_INT = 30;

private:
	std::vector<double> a;
	std::vector<double> w;
	std::vector<int> ip;
	float p[FREQ_NUM];

	std::vector<float> spectru_;
	std::vector<int> spec_;
	std::vector<int> smooth_;

private:
	int times_;
	int pointTimes_;
	float maxSum_;

	std::multiset<int> histry_;
	double rankRatio_;

	bool isEasy_;

};

#endif // ! orchestra_h_

