/**
 * $Id: resource.cc,v 1.14 2003/07/24 10:53:10 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "resource.h"
#include "cpputil.h"
#include "config.h"

#include <SDL.h>

#include <fstream>

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

Resource::Resource() {}

void Resource::addDefault() {
	set("screenWidth", 450);
	set("screenHeight", 600);
	set("screenBpp", 8);
	set("screenFlag", SDL_SWSURFACE);
/*
	set("playerX", 225);
	set("playerY", 500);
	set("playerSpeed", 5);
	set("playerSpeed2", 3);
	set("playerInvisibleTurn", 60);

	set("enemyNum", 7);
	set("enemyCenterX", 225);
	set("enemyCenterY", 150);
	set("enemyEllipseX", 150);
	set("enemyEllipseY", 100);
	set("enemyBaseSpeed", 0.001);

	set("buzzLength", 25);
	set("buzzEffectNum", 15);
	set("collisionLength", 2);
	set("riskyRatio", 1.5);
	set("collisionEffectNum", 100);

	set("song", "music/");

	set("clearMode", 0);

	set("bulletDir", "data/bullets/");
	set("bulletDirNormal", "data/bullets/normal");

	set("scoreDrawScoreX", 10);
	set("scoreDrawScoreY", 9);
	set("scoreDrawBuzzX", 132);
	set("scoreDrawBuzzY", 9);
	set("scoreDrawBuzzScoreX", 217);
	set("scoreDrawBuzzScoreY", 9);
	set("scoreDrawRankX", 332);
	set("scoreDrawRankY", 9);
	set("scoreDrawTimeX", 403);
	set("scoreDrawTimeY", 9);
	set("fpsDrawX", 410);
	set("fpsDrawY", 9);

	set("gameRankNormal", 50);
	set("gameRankEasy", 0);
	set("gameRankMaxNormal", 100);
	set("gameRankMaxEasy", 50);
	set("gameMissRankDownNormal", 20);
	set("gameMissRankDownEasy", 10);

	set("resultLeft", 100);
	set("resultUp", 30);
	set("resultRight", 350);
	set("resultSpace", 20);

	set("cpu", 0);
	set("mode", "normal");
	set("difficulty", "Normal");
*/
}

void Resource::addResourceFile() {
	addResourceFile("data/bpo.conf");
	addResourceFile("save/bpo.conf");
}

void Resource::addAdditionalResourceFile() {
	std::string mode = get<std::string>("mode");
	if (mode != "default") {
		addResourceFile("data/modes/" + mode + ".conf");
	}
}

void Resource::addResourceFile(const std::string& file) {
	std::ifstream is(file.c_str());
	check(is.is_open(), file + ": resource file not found.");

	addResourceStream(is);
}

void Resource::addResourceStream(std::istream& is) {
    static const int MAXBUF = 256;
    char ln[MAXBUF];

    while (is.getline(ln, MAXBUF)) {
		std::string line(ln);
		unsigned int s;

		// Ȥν
		if ((s = line.find("#")) != std::string::npos)
			line.erase(s);

		// Ƭν
		if (!line.empty() && line[0] == ' ')
			line.erase(0, line.find_first_not_of(' '));

		std::istringstream ss(line);
		std::string key, val;
		ss >> key >> val;
		if (val == "=") ss >> val;

		while (ss) {
			std::string add;
			ss >> add;
			if (add != "") val += " " + add;
		}

		if (key != "") {
			setString(key, val);
		}
    }
}

void Resource::addArgument(int argc, char** argv) {
#ifdef HAVE_GETOPT_H
	int optc;
    while ((optc = getopt(argc, argv, "m:c")) != -1) {
        switch (optc) {
		case 'm':
			set("clearMode", optarg);
			break;
		case 'c':
			set("cpu", 1);
			break;
		}
	}
	if (optind < argc) {
		set("song", argv[optind]);
	}
#else
	if (1 < argc) {
		set("song", argv[1]);
	}
#endif
}

void Resource::dump() const {
	for (RcMap::const_iterator ite = resource_.begin();
		 ite != resource_.end(); ++ite)
	{
		std::cout << ite->first << " => " << ite->second.first << std::endl;
	}
}

std::istream& operator>> (std::istream& is, Resource& rc) {
	rc.addResourceStream(is);
	return is;
}

std::ostream& operator<< (std::ostream& os, const Resource& rc) {
	for (Resource::RcMap::const_iterator ite = rc.resource_.begin();
		 ite != rc.resource_.end(); ++ite)
	{
		os << ite->first << " " << ite->second.first << std::endl;
	}
	return os;
}

