/**
 * $Id: scene.cc,v 1.5 2003/04/09 03:09:25 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "scene.h"

#include "scene_title.h"
#include "scene_select.h"
#include "scene_game.h"
#include "scene_result.h"

#include "game.h"
#include "bonus.h"
#include "resource.h"
#include "music.h"

namespace SceneFactory {
	std::auto_ptr<Scene> generateTitle() {
		return std::auto_ptr<Scene>(new SceneTitle());
	}
	std::auto_ptr<Scene> generateSelect() {
		return std::auto_ptr<Scene>(new SceneSelect());
	}
	std::auto_ptr<Scene> generateGame() {
		return std::auto_ptr<Scene>(new SceneGame());
	}
	std::auto_ptr<Scene> generateResult(Game* game) {
		std::auto_ptr<Resource> result(game->getResult().release());
		return std::auto_ptr<Scene>(
			new SceneResult(result, game->getBonus(), game->getMusic()));
	}
	std::auto_ptr<Scene> generateHighscore() {
		std::auto_ptr<Scene> ret(SceneResult::createFromHighscore().release());
		if (ret.get() == 0) {
			return std::auto_ptr<Scene>(generateSelect().release());
		}
		else {
			return ret;
		}
	}
	std::auto_ptr<Scene> generateEnd() {
		return std::auto_ptr<Scene>();
	}

}

std::auto_ptr<Scene> Scene::getEscapedNext() {
	return SceneFactory::generateEnd();
}
