/**
 * $Id: scene_result.cc,v 1.5 2003/06/21 11:08:48 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "scene_result.h"

#include "bpo.h"
#include "bmpfont.h"
#include "padinput.h"
#include "resource.h"
#include "bonus.h"
#include "music.h"

#include <boost/lexical_cast.hpp>

#include <fstream>

SceneResult::SceneResult(std::auto_ptr<Resource> result,
						 std::auto_ptr<BonusMgr> bonus,
						 std::auto_ptr<Music> music)
	: result_(result), bonus_(bonus), music_(music),
	  scr_(BPO::getSurface()), font_(BPO::getBMPFont()),
	  turn_(0), state_(SCORE),
	  resultLeft_(BPO::rc().get<int>("resultLeft")),
	  resultUp_(BPO::rc().get<int>("resultUp")),
	  resultRight_(BPO::rc().get<int>("resultRight")),
	  resultSpace_(BPO::rc().get<int>("resultSpace"))
{}

SceneResult::~SceneResult() {}

std::auto_ptr<Scene> SceneResult::createFromHighscore() {
	std::ifstream is("save/highscore.dat");
	if (!is.is_open()) return std::auto_ptr<Scene>();
	int dummy;
 	is >> dummy;
	Music* music = new Music(is);
	BonusMgr* bonus = new BonusMgr(is);
	Resource* rc = new Resource();
	is >> *rc;

	return std::auto_ptr<Scene>(new SceneResult(
									std::auto_ptr<Resource>(rc),
									std::auto_ptr<BonusMgr>(bonus),
									std::auto_ptr<Music>(music)));
}

bool SceneResult::isEnd() const {
    return state_ == END;
}

std::auto_ptr<Scene> SceneResult::getNext() {
    return SceneFactory::generateTitle();
}

void SceneResult::renderCenter(std::string str, int h) {
	h++;
	int x = (scr_->w - font_->width(str.length())) / 2;
	int y = (h * 10 - turn_ < 0)
		? h * resultSpace_ + resultUp_
		: h * resultSpace_ + resultUp_ - h * 100 + turn_ * 10 ;
	font_->render(str, scr_, x, y);
}

void SceneResult::renderLeft(std::string str, int h) {
	h++;
	int x = (h * 10 - turn_ < 0)
		? resultLeft_
		: resultLeft_ - h * 100 + turn_ * 10;
	int y = h * resultSpace_ + resultUp_;
	font_->render(str, scr_, x, y);
}

void SceneResult::renderRight(std::string str, int h) {
	h++;
	int x = (h * 10 - turn_ < 0)
		? resultRight_ - font_->width(str.length())
		: resultRight_ - font_->width(str.length()) + h * 100 - turn_ * 10;
	int y = h * resultSpace_ + resultUp_;
	font_->render(str, scr_, x, y);
}

void SceneResult::run() {
	turn_++;

	InputBase* input = BPO::getInput();
	if (input->getButton(0) ||
		(BPO::rc().get<bool>("autoStart") && turn_ > 120))
	{
		state_ = static_cast<State_>(state_ + 1);
		if (state_ == MISC) turn_ = 10;
		while (input->getButton(0)) SDL_PumpEvents();
	}

	if (state_ == SCORE) {
		renderCenter("Result", 0);
		renderLeft("Buzz Score", 2);
		renderRight(result_->get<std::string>("buzzCount") + "  Buzz  ", 2);
		renderRight(result_->get<std::string>("buzzScore"), 3);

		int i = 0;
		for (BonusMgr::iterator ite = bonus_->bonuses().begin();
			 ite != bonus_->bonuses().end(); ++ite)
		{
			const Bonus* bonus = *ite;
			renderLeft(bonus->label(), 4+i);
			renderRight(boost::lexical_cast<std::string>(bonus->count()) +
						" Times  ", 4+i);
			renderRight(boost::lexical_cast<std::string>(bonus->allScore()),
						5+i);
			i += 2;
		}

		renderLeft("Score", 21);
		renderRight(result_->get<std::string>("score"), 22);
		renderLeft("Final Score", 24);
		renderRight(result_->get<std::string>("highscored"), 24);
		renderRight(result_->get<std::string>("finalScore"), 25);
	}
	else if (state_ == MISC) {
		renderCenter("Result", 0);
		renderLeft("Time", 2);
		renderRight(result_->get<std::string>("second") + " Second", 3);
		renderLeft("FPS", 4);
		renderRight(result_->get<std::string>("fps"), 5);
		renderLeft("Max Buzz", 6);
		renderRight(result_->get<std::string>("maxBuzz"), 7);
		renderLeft("Buzz Percentage", 8);
		renderRight(result_->get<std::string>("buzzPercent") + " Percent", 9);
		renderLeft("Bullet Per Second", 10);
		renderRight(result_->get<std::string>("bulletPerSecond"), 11);
		renderLeft("Music Name", 12);
		renderRight(music_->name(), 13);
	}
}

