/**
 * $Id: scene_select.cc,v 1.9 2003/07/24 10:53:10 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "scene_select.h"
#include "padinput.h"
#include "bpo.h"
#include "resource.h"
#include "bmpfont.h"
#include "dir_ite.h"

#include <SDL.h>

#include <string>
#include <utility>
#include <vector>
#include <algorithm>

struct SceneSelect::Config_ {
	Config_(const std::string& name0, const std::string& label0)
		: name(name0), label(label0) {}

	std::string name, label;
	std::vector<std::string> elemNames;
	std::vector<std::string> elemLabels;

};

SceneSelect::SceneSelect()
	: scr_(BPO::getSurface()), font_(BPO::getBMPFont()),
	  turn_(0), prevInputTurn_(0), height_(0), end_(false),
	  selectLeft_(BPO::rc().get<int>("selectLeft")),
	  selectUp_(BPO::rc().get<int>("selectUp")),
	  selectRight_(BPO::rc().get<int>("selectRight")),
	  selectSpace_(BPO::rc().get<int>("selectSpace"))
{
	confNames_.push_back("mode");
	confNames_.push_back("difficulty");
	confNames_.push_back("cpu");
	confNames_.push_back("clearMode");
	confNames_.push_back("autoStart");
	confNames_.push_back("bright");

	Config_* c;

	c = new Config_("mode", "Mode");
	Dir modeDir("data/modes");
	for (Dir::iterator ite = modeDir.begin(); ite != modeDir.end(); ++ite) {
		std::string modeName = *ite;
		modeName.erase(modeName.find(".conf"));
		c->elemNames.push_back(modeName);
		c->elemLabels.push_back(modeName);
	}
	confs_.push_back(c);

	c = new Config_("difficulty", "Difficulty");
	c->elemNames.push_back("Normal");
	c->elemLabels.push_back("Normal");
	c->elemNames.push_back("Easy");
	c->elemLabels.push_back("Easy");
	confs_.push_back(c);

	c = new Config_("cpu", "Control");
	c->elemNames.push_back("0");
	c->elemLabels.push_back("Human");
	c->elemNames.push_back("1");
	c->elemLabels.push_back("CPU");
	confs_.push_back(c);

	c = new Config_("clearMode", "Effect Type");
	for (int i = 0; i <= 9; i ++) {
		std::string str(boost::lexical_cast<std::string>(i));
		c->elemNames.push_back(str);
		c->elemLabels.push_back(str);
	}
	confs_.push_back(c);

	c = new Config_("autoStart", "Auto Start");
	c->elemNames.push_back("0");
	c->elemLabels.push_back("OFF");
	c->elemNames.push_back("1");
	c->elemLabels.push_back("ON");
	confs_.push_back(c);

	c = new Config_("bright", "Bright");
	c->elemNames.push_back("1");
	c->elemLabels.push_back("Yes");
	c->elemNames.push_back("0");
	c->elemLabels.push_back("No");
	confs_.push_back(c);

	c = new Config_("highscore", "High Score");
	confs_.push_back(c);

	for (std::vector<Config_*>::const_iterator ite = confs_.begin();
		 ite != confs_.end(); ++ite)
	{
		Config_* conf = *ite;
		if (!conf->elemNames.empty()) {
			size_t d =
				std::distance(conf->elemNames.begin(),
							  std::find(conf->elemNames.begin(),
										conf->elemNames.end(),
										BPO::rc().get<std::string>(conf->name)));
			if (d == conf->elemNames.size()) {
				state_.push_back(0);
			}
			else {
				state_.push_back(d);
			}
		}
	}
}

SceneSelect::~SceneSelect() {
	delete_clear(confs_);
}

bool SceneSelect::isEnd() const {
    return end_;
}

std::auto_ptr<Scene> SceneSelect::getNext() {
	BPO::rc().saveResource("save/bpo.conf",
						   confNames_.begin(), confNames_.end());
	BPO::rc().addAdditionalResourceFile();

	BPO::setBulletMode();

	if (height_ == confs_.size()-1) {
		return SceneFactory::generateHighscore();
	}
	else {
		return SceneFactory::generateGame();
	}
}

void SceneSelect::renderLeft(std::string str, size_t h) {
	h++;
	int x = selectLeft_;
	int y = h * selectSpace_ + selectUp_;
	if (height_ == h-1) font_->renderColored(str, scr_, x, y, 254);
	else font_->render(str, scr_, x, y);
}

void SceneSelect::renderRight(std::string str, size_t h) {
	h++;
	int x = selectRight_ - font_->width(str.length());
	int y = h * selectSpace_ + selectUp_;
	if (height_ == h-1) font_->renderColored(str, scr_, x, y, 254);
	else font_->render(str, scr_, x, y);
}

void SceneSelect::run() {
	turn_++;

	InputBase* input = BPO::getInput();
	if (input->getButton(0) ||
		(BPO::rc().get<bool>("autoStart") && turn_ - prevInputTurn_ > 120))
	{
		end_ = true;
		while (input->getButton(0)) SDL_PumpEvents();
	}

	Axis axis = input->getAxis();

	if (axis != Axis::NONE && prevInputTurn_ + 10 < turn_) {
		prevInputTurn_ = turn_;

		if (axis.isUp() && height_ > 0) height_--;
		if (axis.isDown() && height_ + 1 < confs_.size()) height_++;

		if (!confs_[height_]->elemNames.empty()) {
			if (axis.isRight() &&
				state_[height_] + 1 < confs_[height_]->elemNames.size())
			{
				state_[height_]++;

				Config_* conf = confs_[height_];
				BPO::rc().set(conf->name, conf->elemNames[state_[height_]]);
			}
			if (axis.isLeft() && state_[height_] > 0) {
				state_[height_]--;

				Config_* conf = confs_[height_];
				BPO::rc().set(conf->name, conf->elemNames[state_[height_]]);
			}

			if (height_ == 3) {
				BPO::setClearMode(BPO::rc().get<int>("clearMode"));
			}
			if (height_ == 5) {
				BPO::resetColors();
			}
		}
	}

	size_t i = 0;
	for (std::vector<Config_*>::const_iterator ite = confs_.begin();
		 ite != confs_.end(); ++ite)
	{
		Config_* conf = *ite;
		renderLeft(conf->label, i);
		if (!conf->elemLabels.empty()) {
			renderRight(conf->elemLabels[state_[i]], i);
		}

		i++;
	}

}

