/**
 * $Id: score.cc,v 1.5 2003/04/09 03:09:25 i Exp $
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "score.h"
#include "resource.h"
#include "bmpfont.h"
#include "bpo.h"
#include "mymath.h"

#include "mysdl.h"

#include <iostream>
#include <cstdio>

namespace {
	class StringCache {
	public:
		StringCache(const char* fmtstr);

		char* getString(int val);

	private:
		const char* fmtstr_;
		char buf_[256];
		int prev_;
	};

	StringCache::StringCache(const char* fmtstr)
		: fmtstr_(fmtstr), prev_(-1)
	{}

	char* StringCache::getString(int val) {
		if (prev_ != val) {
			std::sprintf(buf_, fmtstr_, val);
			prev_ = val;
		}
		return buf_;
	}
}

Score::Score()
	: score_(0), buzz_(0), scorePerBuzz_(0), maxBuzz_(0), buzzScore_(0),
	  buzzNum_(0),
	  drawScoreX_(BPO::rc().get<int>("scoreDrawScoreX")),
	  drawScoreY_(BPO::rc().get<int>("scoreDrawScoreY")),
	  drawBuzzX_(BPO::rc().get<int>("scoreDrawBuzzX")),
	  drawBuzzY_(BPO::rc().get<int>("scoreDrawBuzzY")),
	  drawBuzzScoreX_(BPO::rc().get<int>("scoreDrawBuzzScoreX")),
	  drawBuzzScoreY_(BPO::rc().get<int>("scoreDrawBuzzScoreY")),
	  drawRankX_(BPO::rc().get<int>("scoreDrawRankX")),
	  drawRankY_(BPO::rc().get<int>("scoreDrawRankY")),
	  drawTimeX_(BPO::rc().get<int>("scoreDrawTimeX")),
	  drawTimeY_(BPO::rc().get<int>("scoreDrawTimeY"))
{
	surf_ = BPO::getSurface();
	font_ = BPO::getBMPFont();

	scoreSurf_ = MySDL_CreateRGBSurface(SDL_SWSURFACE,
										font_->width(5), font_->height(), 8);
	scoreSurf_ = MySDL_DisplayFormat(scoreSurf_);
	buzzSurf_ = MySDL_CreateRGBSurface(SDL_SWSURFACE,
									   font_->width(4), font_->height(), 8);
	buzzSurf_ = MySDL_DisplayFormat(buzzSurf_);
	rankSurf_ = MySDL_CreateRGBSurface(SDL_SWSURFACE,
									   font_->width(4), font_->height(), 8);
	rankSurf_ = MySDL_DisplayFormat(rankSurf_);
	scorePerBuzzSurf_ =
		MySDL_CreateRGBSurface(SDL_SWSURFACE,
							   font_->width(6), font_->height(), 8);
	scorePerBuzzSurf_ = MySDL_DisplayFormat(scorePerBuzzSurf_);

	font_->render("SCORE", scoreSurf_, 0, 0);
	font_->render("BUZZ", buzzSurf_, 0, 0);
	font_->render("RANK", rankSurf_, 0, 0);
	font_->render("BSCORE", scorePerBuzzSurf_, 0, 0);

	MySDL_SetColorKey(scoreSurf_, SDL_SRCCOLORKEY|SDL_RLEACCEL, 0,0,0);
	MySDL_SetColorKey(buzzSurf_, SDL_SRCCOLORKEY|SDL_RLEACCEL, 0,0,0);
	MySDL_SetColorKey(rankSurf_, SDL_SRCCOLORKEY|SDL_RLEACCEL, 0,0,0);
	MySDL_SetColorKey(scorePerBuzzSurf_, SDL_SRCCOLORKEY|SDL_RLEACCEL, 0,0,0);
}

Score::~Score() {
	SDL_FreeSurface(scoreSurf_);
	SDL_FreeSurface(buzzSurf_);
	SDL_FreeSurface(rankSurf_);
	SDL_FreeSurface(scorePerBuzzSurf_);
}

void Score::doBuzz() {
	buzzNum_++;

	int add = scorePerBuzz_ * rank_ / 10;
	score_ += add;
	buzzScore_ += add;

	buzz_++;

	scorePerBuzz_ += 10 - (scorePerBuzz_ / 100);

	if (maxBuzz_ < buzz_) maxBuzz_ = buzz_;

/*
	std::cout << "score: " << score_ << ", "
			  << "buzz: " << buzz_ << ", "
			  << "buzzscore: " << scorePerBuzz_ << ", "
			  << "maxbuzz: " << maxBuzz_ << std::endl;
*/
}

void Score::doDamage() {
	buzz_ = 0;
	scorePerBuzz_ = 0;

/*
	std::cout << "score: " << score_ << ", "
			  << "buzz: " << buzz_ << ", "
			  << "buzzscore: " << scorePerBuzz_ << ", "
			  << "maxbuzz: " << maxBuzz_ << std::endl;
*/
}

void Score::draw() {
/*
	static int c = 0;
	c ^= 1;
	if (c) return;
*/
	static StringCache scoreStr("% 9d");
	static StringCache buzzStr("% 5d");
	static StringCache rankStr("% 3d");
	static StringCache buzzScoreStr("% 7d");
	int rndx=rnd(0);
	int rndy=rnd(0);

	MySDL_BlitSurface(scoreSurf_, surf_, drawScoreX_+rndx, drawScoreY_+rndy);
	MySDL_BlitSurface(buzzSurf_, surf_, drawBuzzX_+rndx, drawBuzzY_+rndy);
	MySDL_BlitSurface(rankSurf_, surf_, drawRankX_+rndx, drawRankY_+rndy);
	MySDL_BlitSurface(scorePerBuzzSurf_, surf_,
					  drawBuzzScoreX_+rndx, drawBuzzScoreY_+rndy);

	font_->render(scoreStr.getString(score_), surf_,
				  drawScoreX_ + font_->width(5)+rndx, drawScoreY_+rndy);
	font_->render(buzzStr.getString(buzz_), surf_,
				  drawBuzzX_ + font_->width(4)+rndx, drawBuzzY_+rndy);
	font_->render(rankStr.getString(rank_), surf_,
				  drawRankX_ + font_->width(4)+rndx, drawRankY_+rndy);
	font_->render(buzzScoreStr.getString(scorePerBuzz_), surf_,
				  drawBuzzScoreX_+font_->width(6)+rndx, drawBuzzScoreY_+rndy);
}

