/**
 * $Id$
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef singleton_h_
#define singleton_h_

#include "cpputil.h"

template <class T_>
struct CannotNewPolicy {
	static T_* New() { return new T_(); }
};

template <class T_>
struct CanNewPolicy {
	static T_* New() {
		error("singleton yet");
		return 0;     // not reach, but for warning
	}
};

/// 󥰥ȥΥ١
template<class T_, template <class> class NewPolicy_ = CannotNewPolicy>
class Singleton {
public:
	static T_* instance() {
		if (!instance_) {
			instance_ = NewPolicy_<T_>::New();
		}
		return static_cast<T_*>(instance_);
	}

	Singleton() {
		check(instance_ == 0, "Singleton is made twice.\n");
		instance_ = this;
	}

	virtual ~Singleton() {}

protected:
	static Singleton* instance_;
};

template<class T_, template <class> class NewPolicy_>
Singleton<T_, NewPolicy_>* Singleton<T_, NewPolicy_>::instance_ =0;

#endif // ! singleton_h_

